import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

public class Tests {
    Random rnd = new Random(239);

    int MAX_X = 1_000_000_000;
    int MAX_K = 100_000;

    int tests = 1;

    public void writeTest(long X, ArrayList<Long>[] a) {
        a = Arrays.stream(a).filter(x -> x.size() > 0).toArray(ArrayList[]::new);
        long[][] array = new long[a.length][];
        int id = 0;
        for (ArrayList x : a) {
            array[id] = new long[x.size()];
            for (int j = 0; j < array[id].length; j++) {
                array[id][j] = (long) x.get(j);
            }
            id++;
        }
        writeTest(X, array);
    }

    public void writeTest(long X, long[][] a) {
        try {
            PrintWriter out = tests == -1 ? new PrintWriter(System.out) : new PrintWriter(String.format("%02d", tests++));

            out.println(X + " " + a.length);
            for (long[] x : a) {
                out.print(x.length);
                for (long v : x) {
                    out.print(" " + v);
                }
                out.println();
            }

            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sample1() {
        writeTest(1, new long[][]{{-1, 2}, {-2, 3}, {-3, 4}});
    }

    public void sample2() {
        writeTest(1, new long[][]{{-1, -1, 4}, {1, -3, -4, 8}});
    }

    public void corner() {
        writeTest(0, new long[][]{{-1}});
    }

    public void max1() {
        long[][] a = new long[MAX_K][1];
        for (int i = 0; i < a.length; i++) {
            a[i][0] = MAX_X;
        }
        writeTest(MAX_X, a);
    }

    public void max2() {
        long[][] a = new long[1][MAX_K];
        Arrays.fill(a[0], MAX_X);
        writeTest(MAX_X, a);
    }

    public void max3() {
        long[][] a = new long[MAX_K / 2][2];
        for (int i = 0; i < a.length; i++) {
            a[i][0] = -(MAX_X - 1);
            a[i][1] = MAX_X;
        }
        writeTest(MAX_X, a);
    }

    public void max4() {
        long[][] a = new long[1][MAX_K];
        for (int i = 0; i < a[0].length; i++) {
            if (i < a[0].length / 2) {
                a[0][i] = -(MAX_X - 1);
            } else {
                a[0][i] = MAX_X;
            }
        }
        writeTest(MAX_X, a);
    }

    public void max5() {
        long[][] a = new long[1][MAX_K];
        for (int i = 0; i < a[0].length; i++) {
            if (i % 2 == 0) {
                a[0][i] = -(MAX_X - 1);
            } else {
                a[0][i] = MAX_X;
            }
        }
        writeTest(MAX_X, a);
    }

    public void max6() {
        ArrayList<Long>[] a = new ArrayList[1];
        a[0] = new ArrayList<>();
        for (int i = 0; i < MAX_K; i++) {
            if (i % 2 == 0) {
                a[0].add((long) (-(MAX_X / 10 - 1)));
            } else {
                a[0].add((long) (MAX_X / 10));
            }
        }
        Collections.shuffle(a[0], rnd);
        writeTest(MAX_X, a);
    }

    public void takeAllRandom(int MAX_K, int rows) {
        int size = rnd.nextInt(rows) + 1;
        ArrayList<Long>[] a = new ArrayList[size];
        for (int i = 0; i < a.length; i++) {
            a[i] = new ArrayList<>();
        }

        for (long i = 0; i < MAX_K / 2; i++) {
            int id = rnd.nextInt(size);
            a[id].add(-(i + 1));
            a[id].add(i + 2);
        }
        writeTest(1, a);
    }

    public void takePartRandom(int MAX_K, int rows) {
        int size = rnd.nextInt(rows) + 1;
        ArrayList<Long>[] a = new ArrayList[size];
        for (int i = 0; i < a.length; i++) {
            a[i] = new ArrayList<>();
        }
        int use = rnd.nextInt(MAX_K / 2) + 1;
        for (long i = 0; i < use / 2; i++) {
            int id = rnd.nextInt(size);
            a[id].add(-(i + 1));
            a[id].add(i + 2);
        }
        int MAX_V = 10_000_000;
        for (int i = use; i < MAX_K; i++) {
            int id = rnd.nextInt(size);
            long v = MAX_V / 2 - rnd.nextInt(MAX_V);
            a[id].add(v);
        }
        writeTest(1, a);
    }

    public void fullRandom(int MAX_K, int rows) {
        int size = rnd.nextInt(rows) + 1;
        ArrayList<Long>[] a = new ArrayList[size];
        for (int i = 0; i < a.length; i++) {
            a[i] = new ArrayList<>();
        }
        int MAX_V = 100;
        int slices = Math.min(7, MAX_K / 5);
        long X = MAX_V;
        for (int slice = 0; slice < slices; slice++) {
            for (int i = 0; i < MAX_K / slices; i++) {
                int id = rnd.nextInt(size);
                a[id].add((long) MAX_V / 2 - rnd.nextInt(MAX_V));
            }
            MAX_V *= 10;
        }
        writeTest(X, a);
    }

    public void gen(String[] args, int test) {
        tests = test;
        switch (args[0]) {
            case "sample1":
                sample1();
                break;
            case "sample2":
                sample2();
                break;
            case "max1":
                max1();
                break;
            case "max2":
                max2();
                break;
            case "max3":
                max3();
                break;
            case "max4":
                max4();
                break;
            case "max5":
                max5();
                break;
            case "max6":
                int seed = Integer.parseInt(args[1]);
                rnd = new Random(seed);
                max6();
                break;
            case "takeAllRandom":
                int k = Integer.parseInt(args[1]);
                int rows = Integer.parseInt(args[2]);
                seed = Integer.parseInt(args[3]);
                rnd = new Random(seed);
                takeAllRandom(k, rows);
                break;
            case "takePartRandom":
                k = Integer.parseInt(args[1]);
                rows = Integer.parseInt(args[2]);
                seed = Integer.parseInt(args[3]);
                rnd = new Random(seed);
                takePartRandom(k, rows);
                break;
            case "fullRandom":
                k = Integer.parseInt(args[1]);
                rows = Integer.parseInt(args[2]);
                seed = Integer.parseInt(args[3]);
                rnd = new Random(seed);
                fullRandom(k, rows);
                break;
            case "corner":
                corner();
                break;
        }
    }

    public void gen() {
        try {
            Scanner in = new Scanner(new File("tests.txt"));
            int test = 1;
            while (in.hasNext()) {
                String line = in.nextLine();
                if (line.length() == 0)
                    break;
                String[] x = line.split(" ");
                gen(Arrays.copyOfRange(x, 1, x.length), test++);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        if (args.length > 0) {
            new Tests().gen(args, -1);
        } else {
            new Tests().gen();
        }
    }
}