#define _CRT_SECURE_NO_WARNINGS
#include "testlib.h"
#include <unordered_map>
#include <unordered_set>
#include <algorithm>
#include <iostream>
#include <cstring>
#include <cassert>
#include <complex>
#include <chrono>
#include <random>
#include <bitset>
#include <cstdio>
#include <vector>
#include <string>
#include <stack>
#include <tuple>
#include <queue>
#include <ctime>
#include <cmath>
#include <list>
#include <map>
#include <set>

using namespace std;

struct Test {
  int n;
  vector < pair<int, int> > cactus1, cactus2;
};

int main(int argc, char **argv) {
  registerGen(argc, argv, 1);

  int start = atoi(argv[1]);

  vector <Test> tests;

  auto ShuffleCactus = [&](vector < pair<int, int> > &cactus) {
    for (int i = 0; i < (int)cactus.size(); i++) {
      if (rnd.next(2)) {
        swap(cactus[i].first, cactus[i].second);
      }
    }
    shuffle(cactus.begin(), cactus.end());
  };

  auto Add = [&](Test test) {
    ShuffleCactus(test.cactus1);
    ShuffleCactus(test.cactus2);
    tests.push_back(test);
  };

  auto EqualDistribution = [&](int total_cnt, int total_sum) {
    vector <int> answ;

    for (int i = 0; i < total_cnt; i++) {
      if (i < total_sum % total_cnt) {
        answ.push_back(total_sum / total_cnt + 1);
      } else {
        answ.push_back(total_sum / total_cnt);
      }
    }

    return answ;
  };

  auto RandomDistribution = [&](int total_cnt, int total_sum) {
    vector <int> answ;

    for (int i = 0; i < total_cnt; i++) {
      int val = (i == total_cnt - 1) ? total_sum : rnd.next(total_sum + 1);
      answ.push_back(val);
      total_sum -= val;
    }

    assert(total_sum == 0);

    return answ;
  };

  auto GenerateCyclesAndBridges = [&](int n, int m, int total_bridges, function<vector<int>(int, int)> distribution) {
    int total_cycles = m - (n - 1);
    assert(n >= 1 + 2 * total_cycles);

    if (total_bridges == -1) {
      if (m == n - 1) {
        total_bridges = n - 1;
      } else {
        total_bridges = rnd.next(n - (1 + 2 * total_cycles) + 1);
      }
    } else {
      total_bridges = min(total_bridges, n - (1 + 2 * total_cycles));
    }

    // Let's consider that we have an integer `x` in the array `groups` `cnt` times 
    // If `x` is equal to `1`, then we should have exactly `cnt` bridges in the generated cactus
    // If `x` is greater than `1`, then we should have exactly `cnt` cycles of length `x + 1` in the generated cactus
    vector <int> groups;
    for (int i = 1; i <= total_bridges; i++) {
      groups.push_back(1);
    }

    int remaining = n - (1 + 2 * total_cycles) - total_bridges;
    vector <int> cycles_lengths = distribution(total_cycles, remaining);
    for (const auto& cur: cycles_lengths) {
      groups.push_back(cur + 2);
    }

    shuffle(groups.begin(), groups.end());

    return groups;
  };

  auto GenerateRandomOrder = [&](int n) {
    vector <int> vertices;
    for (int i = 1; i <= n; i++) {
      vertices.push_back(i);
    }
    shuffle(vertices.begin(), vertices.end());
    return vertices;
  };

  auto GenerateRandomCactus = [&](int n, int m, int total_bridges, function<vector<int>(int, int)> distribution) {
    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> groups = GenerateCyclesAndBridges(n, m, total_bridges, distribution);

    vector <int> available_vertices = {vertices[0]};
    int it = 1;

    vector < pair<int, int> > cactus;

    for (auto cur_group: groups) {
      int parent = available_vertices[ rnd.next((int)available_vertices.size()) ];

      if (cur_group == 1) {
        cactus.emplace_back(parent, vertices[it]);
        available_vertices.push_back(vertices[it++]);
      } else {
        int sz = cur_group + 1;

        vector <int> cycle = {parent};
        while (cur_group--) {
          cycle.push_back(vertices[it]);
          available_vertices.push_back(vertices[it++]);
        }

        assert(sz >= 3);

        for (int i = 0; i < sz; i++) {
          cactus.emplace_back(cycle[i], cycle[(i + 1) % sz]);
        }
      }
    }

    return cactus;
  };

  auto GenerateChainCactus = [&](int n, int m, int total_bridges, function<vector<int>(int, int)> distribution) {
    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> groups = GenerateCyclesAndBridges(n, m, total_bridges, distribution);

    vector <int> available_vertices = {vertices[0]};
    int it = 1;

    vector < pair<int, int> > cactus;

    for (auto cur_group: groups) {
      int parent = available_vertices[ rnd.next((int)available_vertices.size()) ];
      available_vertices.clear();

      if (cur_group == 1) {
        cactus.emplace_back(parent, vertices[it]);
        available_vertices.push_back(vertices[it++]);
      } else {
        int sz = cur_group + 1;

        vector <int> cycle = {parent};
        while (cur_group--) {
          cycle.push_back(vertices[it]);
          available_vertices.push_back(vertices[it++]);
        }

        assert(sz >= 3);

        for (int i = 0; i < sz; i++) {
          cactus.emplace_back(cycle[i], cycle[(i + 1) % sz]);
        }
      }
    }

    return cactus;
  };

  auto GenerateStarCactus = [&](int n, int m, int total_bridges, function<vector<int>(int, int)> distribution) {
    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> groups = GenerateCyclesAndBridges(n, m, total_bridges, distribution);

    vector <int> available_vertices = {vertices[0]};
    int it = 1;

    vector < pair<int, int> > cactus;

    for (auto cur_group: groups) {
      int parent = vertices[0];

      if (cur_group == 1) {
        cactus.emplace_back(parent, vertices[it++]);
      } else {
        int sz = cur_group + 1;

        vector <int> cycle = {parent};
        while (cur_group--) {
          cycle.push_back(vertices[it++]);
        }

        assert(sz >= 3);

        for (int i = 0; i < sz; i++) {
          cactus.emplace_back(cycle[i], cycle[(i + 1) % sz]);
        }
      }
    }

    return cactus;
  };

  auto GenerateGoodRandomCactus = [&](int n, int m, int total_bridges, function<vector<int>(int, int)> distribution) {
    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> groups = GenerateCyclesAndBridges(n, m, total_bridges, distribution);

    int sum = 0;
    for (int x: groups) {
      sum += x;
    }
    assert(sum == n - 1);

    vector <int> available_vertices = {vertices[0]};
    int it = 1;

    vector < pair<int, int> > cactus;

    for (auto cur_group: groups) {
      int parent = available_vertices[ rnd.next((int)available_vertices.size()) ];

      if (cur_group == 1) {
        cactus.emplace_back(parent, vertices[it]);
        available_vertices.push_back(vertices[it++]);
      } else {
        int sz = cur_group;

        vector <int> cycle = {parent};
        while (sz--) {
          cycle.push_back(vertices[it]);
          available_vertices.push_back(vertices[it++]);
        }

        sz = cur_group + 1;

        assert(sz >= 3);

        for (int i = 0; i < sz; i++) {
          cactus.emplace_back(cycle[i], cycle[(i + 1) % sz]);
        }
      }

      reverse(available_vertices.begin(), available_vertices.end());
      int final_sz = min(2 * cur_group, (int)available_vertices.size());
      while ((int)available_vertices.size() > final_sz) {
        available_vertices.pop_back();
      }
      reverse(available_vertices.begin(), available_vertices.end());
    }

    return cactus;
  };

  // small manual tests
  Add({
    3,
    {{1, 2}, {1, 3}, {3, 2}},
    {{3, 1}, {2, 1}, {2, 3}}
  });
  Add({
    5,
    {{1, 2}, {1, 3}, {1, 4}, {1, 5}},
    {{2, 1}, {3, 2}, {3, 4}, {5, 4}}
  });

  vector <int> nn = {20, 21, 50, 51, 150, 151, 300, 301, 500, 501, 700, 701, 999, 1000};

  for (int n: nn) {
    int minm = n - 1;
    int maxm = 3 * (n - 1) / 2;

    {
      // exactly one cycle -> cactus containing one cycle
      auto cactus1 = GenerateRandomCactus(n, n, 0, EqualDistribution);
      auto cactus2 = GenerateRandomCactus(n, n, n / 2, EqualDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      // two random trees
      auto cactus1 = GenerateRandomCactus(n, n - 1, n - 1, EqualDistribution);
      auto cactus2 = GenerateRandomCactus(n, n - 1, n - 1, EqualDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      // two random cactuses
      auto cactus1 = GenerateRandomCactus(n, n + n / 4, -1, RandomDistribution);
      auto cactus2 = GenerateRandomCactus(n, n + n / 4, -1, RandomDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      // two no bridges
      auto cactus1 = GenerateRandomCactus(n, n + n / 3, 0, EqualDistribution);
      auto cactus2 = GenerateRandomCactus(n, n + n / 3, 0, RandomDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      // same cactus
      auto cactus = GenerateRandomCactus(n, n + n / 5, -1, RandomDistribution);
      Add({n, cactus, cactus});
    }
    {
      // no bridge -> cycles with length 3 and bridges
      auto cactus1 = GenerateStarCactus(n, n + 3 * n / 8, 0, RandomDistribution);
      auto cactus2 = GenerateChainCactus(n, n + 3 * n / 8, 2 * n, RandomDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      auto cactus1 = GenerateStarCactus(n, maxm, 0, RandomDistribution);
      auto cactus2 = GenerateChainCactus(n, maxm, 0, RandomDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      int counter = 4;
      while (counter--) {
        int cur_m = rnd.next(minm, maxm);
        auto cactus1 = GenerateGoodRandomCactus(n, cur_m, -1, RandomDistribution);
        auto cactus2 = GenerateGoodRandomCactus(n, cur_m, -1, RandomDistribution);
        Add({n, cactus1, cactus2});
      }
    }
    {
      int counter = 4;
      while (counter--) {
        int cur_m = rnd.next(minm, maxm);
        auto cactus1 = GenerateChainCactus(n, cur_m, -1, RandomDistribution);
        auto cactus2 = GenerateChainCactus(n, cur_m, -1, EqualDistribution);
        Add({n, cactus1, cactus2});
      }
    }
    {
      int counter = 4;
      while (counter--) {
        int cur_m = rnd.next(minm, maxm);
        auto cactus1 = GenerateStarCactus(n, cur_m, -1, EqualDistribution);
        auto cactus2 = GenerateRandomCactus(n, cur_m, -1, RandomDistribution);
        Add({n, cactus1, cactus2});
      }
    }

    if (n % 2 == 0) continue;

    // cactuses containing only cycles of length 3 and there are no bridges

    {
      // same cactus
      auto cactus = GenerateGoodRandomCactus(n, maxm, 0, EqualDistribution);
      Add({n, cactus, cactus});
    }
    {
      // answer is "NO"
      auto cactus1 = GenerateGoodRandomCactus(n, maxm, 0, EqualDistribution);
      auto cactus2 = GenerateRandomCactus(n, maxm, 0, EqualDistribution);
      Add({n, cactus1, cactus2});
    }
    {
      // answer is "NO"
      auto cactus1 = GenerateChainCactus(n, maxm, 0, EqualDistribution);
      auto cactus2 = GenerateStarCactus(n, maxm, 0, EqualDistribution);
      Add({n, cactus1, cactus2});
    }
  }

  for (auto& t : tests) {
    startTest(start++);

    int m = (int)t.cactus1.size();
    cout << t.n << " " << m << endl;
    for (int i = 0; i < m; i++) {
      cout << t.cactus1[i].first << " " << t.cactus1[i].second << endl;
    }
    for (int i = 0; i < m; i++) {
      cout << t.cactus2[i].first << " " << t.cactus2[i].second << endl;
    }
  }
  return 0;
}
