#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

constexpr int MAX_LENGTH = 1000;

class big_int {
public:
    explicit big_int(string digits_string) {
    	if (digits_string[0] == '-') {
    		digits_string = digits_string.substr(1, digits_string.size());
    		sign = -1;
    	} else {
    		sign = 1;
    	}
        size_t end_idx = digits_string.size();
        while (end_idx > 0) {
            size_t start_idx;
            if (end_idx >= BASE_LENGTH) {
                start_idx = end_idx - BASE_LENGTH;
            } else {
                start_idx = 0;
            }
            string chunk = digits_string.substr(start_idx, end_idx - start_idx);
            digits.push_back(stoi(chunk));
            end_idx = start_idx;
        }
        normalize();
    }

    explicit big_int(const char *a) : big_int(string(a)) {}

    big_int(long long value) : big_int(std::to_string(value)) {}

    big_int& operator+=(const big_int& other) {
        if (sign != other.sign) {
            *this -= (-other);
        } else {
            size_t carry = 0;
            for (size_t pos = 0; pos < other.digits.size() || carry > 0; pos++) {
                if (pos >= digits.size()) digits.push_back(0);
                digits[pos] += carry;
                if (pos < other.digits.size()) {
                    digits[pos] += other.digits[pos];
                }
                if (digits[pos] >= BASE) {
                    digits[pos] -= BASE;
                    carry = 1;
                } else {
                    carry = 0;
                }
            }
        }
        normalize();
        return *this;
    }

    big_int& operator-=(const big_int& other) {
        if (other.digits.size() == 0) return *this;
        if (sign != other.sign) {
            *this += (-other);
        } else {
            if ((sign == 1 && other <= *this) || (sign == -1 && *this <= other)) {
                int carry = 0;
                for (size_t pos = 0; pos < other.digits.size() || carry != 0; pos++) {
                    digits[pos] -= carry;
                    if (pos < other.digits.size()) {
                        digits[pos] -= other.digits[pos];
                    }
                    if (digits[pos] < 0) {
                        digits[pos] += BASE;
                        carry = 1;
                    } else {
                        carry = 0;
                    }
                }
            } else {
                *this = other - *this;
                this->sign = -this->sign;
            }
        }
        normalize();
        return *this;
    }

    big_int& operator*=(const big_int& other) {
        vector<int> new_digits(digits.size() + other.digits.size(), 0);
        for (size_t i = 0; i < digits.size(); i++) {
            for (size_t j = 0; j < other.digits.size(); j++) {
                long long mul = (long long) digits[i] * other.digits[j];
                for (size_t pos = i + j; mul > 0; pos++) {
                    mul += new_digits[pos];
                    new_digits[pos] = mul % BASE;
                    mul /= BASE;
                }                
            }
        }
        swap(this->digits, new_digits);
        if (other.sign == -1) {
            sign = -sign;
        }
        normalize();
        return *this; 
    }

    big_int operator+(const big_int& other) const {
        big_int result = *this;
        result += other;
        return result;
    }

    big_int operator-(const big_int& other) const {
        big_int result = *this;
        result -= other;
        return result;
    }

    big_int operator-() const {
        big_int copy = *this;
        copy.sign = -copy.sign;
        copy.normalize();
        return copy;
   }
    
    string to_string() const {
        if (digits.empty()) {
            return "0";
        }
        string output = "";
        if (sign == -1) {
            output += "-";
        }
        for (auto it = digits.rbegin(); it != digits.rend(); ++it) {
            output += format_digit(*it, it != digits.rbegin());
        }
        return output;
    }

    bool operator<= (const big_int& other) const {
        if (sign != other.sign) {
            return sign < other.sign;
        }
        int cmp = compare_signless(other);
        return (sign == 1 && cmp <= 0) || (sign == -1 && cmp >= 0);
    }

    bool operator==(const big_int &other) const { return sign == other.sign && digits == other.digits; }

    bool operator!=(const big_int &other) const { return !(*this == other); }

private:
    static constexpr int BASE = 1'000'000'000;
    static constexpr int BASE_LENGTH = 9;
    std::vector<int> digits;
    int sign;

    static string format_digit(int digit, bool pad = false) {
        std::ostringstream out;
        if (pad) {
            out << std::right << std::setfill('0') << std::setw(BASE_LENGTH);
        }
        out << digit;
        return out.str();
    }

    void normalize() {
        while (digits.size() > 0 && digits.back() == 0) {
            digits.pop_back();
        }
        if (digits.empty() && sign == -1) {
            sign = 1;
        }
    }

    static int compare(size_t a, size_t b) {
        if (a < b) return -1;
        if (a == b) return 0;
        return 1;
    }

    int compare_signless(const big_int& other) const {
        if (digits.size() != other.digits.size()) {
            return compare(digits.size(), other.digits.size());
        }
        if (digits.empty()) {
            return 0;
        }
        for (size_t i = digits.size() - 1; i >= 0; i--) {
            if (digits[i] != other.digits[i]) {
                return compare(digits[i], other.digits[i]);
            }
        }
        return 0;
    }
};

ostream& operator<<(ostream& os, const big_int& value) {
    os << value.to_string();
    return os;
}

string expr;
int pos;

big_int parse_number(InStream& ouf) {
    if (pos == expr.size()) {
        ouf.quitf(_wa, "Expected a number, but reached end of line"); 
    }
    string digits = "";
    int new_pos = pos;
    while (new_pos < expr.size() && '0' <= expr[new_pos] && expr[new_pos] <= '9') {
        digits += expr[new_pos++];
    }
    if (digits.empty()) {
        ouf.quitf(_wa, "Expected a digit at position %d, but got %c", pos, expr[pos]);
    }
    if (digits.size() > 1 && digits[0] == '0') {
        ouf.quitf(_wa, "The number starting at position %d has a leading zero", pos);
    }
    pos = new_pos;
    return big_int(digits);
}

big_int parse_term(InStream& ouf) {
    big_int result = parse_number(ouf);
    while (pos < expr.size() && expr[pos] == '*') {
        pos++;
        big_int term = parse_term(ouf);
        result *= term;
    }
    return result;
}

big_int parse_expr(InStream& ouf) {
    big_int result = parse_term(ouf);
    while (pos < expr.size() && (expr[pos] == '+' || expr[pos] == '-')) {
        char op = expr[pos];
        pos++;
        big_int term = parse_term(ouf);
        if (op == '+') {
            result += term;
        } else {
            result -= term;
        }
    }
    return result;
}

big_int eval(string expr, InStream& ouf) {
    ::expr = expr;
    ::pos = 0;
    big_int result = parse_expr(ouf);
    if (pos != expr.size()) {
        ouf.quitf(_wa, "Could not parse the entire expression, stopped at %d", pos);
    }
    return result;
}

string repr(const big_int& value) {
    string as_str = value.to_string();
    if (as_str.size() > 50) {
        as_str = as_str.substr(0, 25) + "..." + as_str.substr(as_str.size() - 25, 25);
    }
    return as_str;
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);
	long long p = inf.readLong();
	long long q = inf.readLong();
    string expr = lowerCase(ouf.readLine(format("[0-9+\\-*]+"), "expr"));
    if (expr.size() > MAX_LENGTH) {
        quitf(_wa, "The length of the expression exceeds the limit: %d > %d", expr.size(), MAX_LENGTH);
    }
    big_int resP = eval(expr, ouf);
	if (resP != p) {
		quitf(_wa, "eval(expr) must be equal to %lld, but is equal to %s instead", p, repr(resP).c_str());
	}
	reverse(expr.begin(), expr.end());
	big_int resQ = eval(expr, ouf);
	if (resQ != q) {
		quitf(_wa, "eval(expr.reversed()) must be equal to %lld, but is equal to %s instead", q, repr(resQ).c_str()); 
	}
	quitf(_ok, "p = %lld, q = %lld, length = %d", p, q, expr.size());
}