#include <algorithm>
#include <cassert>
#include <iostream>
#include <sstream>
#include <vector>

using namespace std;
using LL = long long;

string solve_eq(LL x) {
    static const vector<LL> atoms = [] {
        vector<LL> res; // 1, 2, ..., 9, 11, 22, ..., 99, 111, 222, ..., 999, ...
        LL first = 1; // 1, 11, 111, ...
        for (int len = 1; len <= 19; len++) {
            for (int d = 1; d < 10; d++) {
                if (len == 19 && d >= 5) {
                    break;
                }
                res.push_back(first * d);
            }
            first = first * 10 + 1;
        }
        return res;
    }();

    assert(x > 0);
    string res;
    while (x > 0) {
        auto larger = upper_bound(atoms.begin(), atoms.end(), x);
        assert(larger > atoms.begin());
        LL add = *prev(larger);
        assert(1 <= add && add <= x);
        if (!res.empty()) {
            res += "+";
        }
        res += to_string(add);
        x -= add;
    }

    return res;
}

struct ret_t {
    string res{};
    LL value{0};
    LL rev_value{0};
};

ret_t solve_big_delta(LL delta) {
    assert(delta >= 9);

    int k = 0;
    int d = -1;
    LL nine = 1;
    while (delta >= nine * 9 && delta >= (double) nine * 8) {
        nine *= 9;
        k++;
    }
    assert(delta >= nine);
    d = delta / nine;
    assert(delta >= nine * d);
    assert(k >= 1);
    assert(1 <= d && d <= 8);

    // delta >= d * 9^k

    LL prefix = 10 * (d + 1) + 1;
    LL rev_suffix = 10 + (d + 1);
    string res = to_string(prefix);
    for (int i = 0; i < k - 1; i++) {
        res += "*9";
    }

    return ret_t{
        .res = std::move(res),
        .value = prefix * (nine / 9),
        .rev_value = rev_suffix * (nine / 9),
    };
}

ret_t solve_small_delta(LL delta) {
    assert(delta > 0 && delta < 9);

    ret_t ret{};

    if (delta % 2 == 1) {
        ret.res = "9-21";
        ret.value = 9 - 21; // -12
        ret.rev_value = 12 - 9; // 3
        delta -= (ret.value - ret.rev_value); // delta += 15
    }

    assert(delta >= 0);
    assert(delta % 2 == 0);

    while (delta > 0) {
        if (!ret.res.empty()) {
            ret.res += "+";
        }
        ret.res += "1-0+1";
        ret.value += 1 - 0 + 1; // 2
        ret.rev_value += 1 + 0 - 1; // 0
        delta -= 2;
    }

    assert(delta == 0);
    return ret;
}

string solve(LL p, LL q) {
    if (p < q) {
        auto res = solve(q, p);
        reverse(res.begin(), res.end());
        return res;
    }

    if (p == q) {
        if (p == 0) {
            return "0";
        }
        auto res = solve_eq(abs(p));
        if (p > 0) {
            return res;
        }
        for (char& x : res) {
            if (x == '+') {
                x = '-';
            }
        }
        return "0-" + res + "-0";
    }

    string res;
    for (int iter = 0; p > q; iter++) {
        assert(iter < 1000); // to prevent infinite cycle in case of bug

        const LL delta_before = p - q;
        assert(delta_before > 0);

        ret_t cur = (delta_before >= 9)
            ? solve_big_delta(delta_before)
            : solve_small_delta(delta_before);

        if (!res.empty()) {
            res += "+";
        }
        res += cur.res;
        p -= cur.value;
        q -= cur.rev_value;

        const LL delta_after = p - q;
        assert(delta_after >= 0);
        assert(delta_after < delta_before);
    }

    assert(p == q);
    assert(!res.empty());
    res += "+" + solve(p, q);

    return res;
}

int main() {
    std::ios::sync_with_stdio(false);

    LL p, q;
    cin >> p >> q;

    auto ans = solve(p, q);
    cout << ans << endl;
}
