#include "testlib.h"

std::pair<std::vector<std::pair<int, int>>, int> generate_random_tree(char* argv[]) {
    const int n_min = atoi(argv[2]);
    const int n_max = atoi(argv[3]);
    const int elongation_min = atoi(argv[4]);
    const int elongation_max = atoi(argv[5]);
    
    const int n = rnd.next(n_min, n_max);
    const int elongation = rnd.next(elongation_min, elongation_max);
    std::vector<std::pair<int, int>> edges(n - 1);
    for (int i = 0; i < n - 1; ++i) {
        edges[i].first = i + 1;
        edges[i].second = rnd.wnext(i + 1, elongation);
    }
    return {edges, rnd.next(n)};
}

std::pair<std::vector<std::pair<int, int>>, int> generate_double_star(char* argv[]) {
    const int n = atoi(argv[2]);
    const int k = atoi(argv[3]);
    const int l = atoi(argv[4]);
    // 0..k - 1: left star
    // k..n-l-1: path
    // n-l..n-1: right star
    ensuref(k >= 0, "Incorrect value of k: %d", k);
    ensuref(l >= 0, "Incorrect value of l: %d", l);
    ensuref(n - k - l >= 1, "Incorrect value of n - k - l: %d", n - k - l);
    std::vector<std::pair<int, int>> edges(n - 1);
    for (int i = 0; i < k; ++i)
        edges[i] = {i, k};
    for (int i = k; i < n - l - 1; ++i)
        edges[i] = {i, i + 1};
    for (int i = n - l - 1; i < n - 1; ++i)
        edges[i] = {n - l - 1, i + 1};
    return {edges, rnd.next(n)};
}

std::pair<std::vector<std::pair<int, int>>, int> generate_binary_tree(char* argv[]) {
    const int n_min = atoi(argv[2]);
    const int n_max = atoi(argv[3]);
    const int n = rnd.next(n_min, n_max);
    std::vector<std::pair<int, int>> edges(n - 1);
    for (int i = 0; i < n - 1; ++i) {
        edges[i].first = i + 1;
        edges[i].second = i / 2;
    }
    return {edges, rnd.next(n)};
}

std::pair<std::vector<std::pair<int, int>>, int> generate_test(char* argv[]) {
    std::string test_type = argv[1];
    if (test_type == "random")
        return generate_random_tree(argv);
    else if (test_type == "double_star")
        return generate_double_star(argv);
    else if (test_type == "binary_tree")
        return generate_binary_tree(argv);
    ensuref(false, "Unknown test type: \"%s\"", argv[1]);
}

std::pair<std::vector<std::pair<int, int>>, int> permuted(std::pair<std::vector<std::pair<int, int>>, int> test) {
    const int n = test.first.size() + 1;
    std::vector<int> permutation(n);
    for (int i = 0; i < n; ++i)
        permutation[i] = i;
    shuffle(permutation.begin() + 1, permutation.end());
    
    for (std::pair<int, int>& edge : test.first) {
        edge.first = permutation[edge.first];
        edge.second = permutation[edge.second];
        if (rnd.next(2))
            std::swap(edge.first,  edge.second);
    }
    shuffle(test.first.begin(), test.first.end());
    test.second = permutation[test.second];
    return test;
}

std::pair<std::vector<std::pair<int, int>>, int> generate_random_test(char* argv[]) {
    return permuted(generate_test(argv));
}

void print_test(const std::pair<std::vector<std::pair<int, int>>, int>& test) {
    std::cout << test.first.size() + 1 << '\n';
    for (const std::pair<int, int>& edge : test.first)
        std::cout << edge.first + 1 << ' ' << edge.second + 1 << '\n';
    std::cout << test.second + 1 << '\n';
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    print_test(generate_random_test(argv));
}
