#include <iostream>
#include <vector>
#include <iomanip>
#include <limits>
 
using namespace std;
 
using Float = long double;
 
vector<Float> find_leaves_ans(const vector<vector<int>>& distances, const vector<int>& leaves) {
    vector<Float> ans(distances.size(), -1);
    vector<Float> temp(distances.size(), -1);
    for (int i : leaves)
        ans[i] = 1;
    for (int iter = 0; iter < 15000; ++iter) {
        ans.swap(temp);
        Float numerator = 0;
        for (int i : leaves) {
            numerator += temp[i];
            ans[i] = leaves.size() - 1;
            for (int j : leaves)
                ans[i] += distances[i][j] * temp[j];
        }
        for (int i : leaves)
            ans[i] = numerator / ans[i];
    }
    for (int i : leaves)
        ans[i] = 1 / ans[i];
    return ans;
}
 
vector<Float> find_ans(const vector<vector<int>>& distances, const vector<int>& leaves) {
    if (leaves.size() <= 1)
        return vector<Float>(distances.size());
    vector<Float> ans = find_leaves_ans(distances, leaves);
    Float denominator = 0;
    for (int i : leaves)
        denominator += 1 / ans[i];
    for (int i = 0; i < distances.size(); ++i)
        if (ans[i] < 0) {
            Float numerator = leaves.size() - 1;
            for (int j : leaves)
                numerator += distances[i][j] / ans[j];
            ans[i] = numerator / denominator;
        }
    return ans;
}
 
void dfs_distances(const int v, const vector<vector<int>>& g, vector<int>& distances) {
    for (int to : g[v])
        if (distances[to] == -1) {
            distances[to] = 1 + distances[v];
            dfs_distances(to, g, distances);
        }
}
 
vector<int> find_distances(const vector<vector<int>>& g, const int v) {
    vector<int> d(g.size(), -1);
    d[v] = 0;
    dfs_distances(v, g, d);
    return d;
}
 
int main() {
    int n;
    cin >> n;
    vector<vector<int>> g(n);
    for (int i = 0; i < n - 1; ++i) {
        int x, y;
        cin >> x >> y;
        --x;
        --y;
        g[x].push_back(y);
        g[y].push_back(x);
    }
    vector<vector<int>> distances(n);
    for (int i = 0; i < n; ++i)
        distances[i] = find_distances(g, i);
    vector<int> leaves;
    for (int i = 0; i < n; ++i)
        if (g[i].size() == 1)
            leaves.push_back(i);
    vector<Float> ans = find_ans(distances, leaves);
    int s;
    cin >> s;
    --s;
    cout << setprecision(30) << fixed << ans[s] << '\n';
}
