class Tree:
    def __init__(self, edges: list) -> None:
        self.adjacency = [[] for i in range(len(edges) + 1)]
        for u, v in edges:
            self.adjacency[u].append(v)
            self.adjacency[v].append(u)
    
    def n(self) -> int:
        return len(self.adjacency)
    
    def dfs_distances(self, v: int, distances: list) -> None:
        for to in self.adjacency[v]:
            if distances[to] == -1:
                distances[to] = 1 + distances[v]
                self.dfs_distances(to, distances)
    
    def find_distances(self, root: int) -> list:
        distances = [-1] * self.n()
        distances[root] = 0
        self.dfs_distances(root, distances)
        return distances
    
    def find_leaves(self) -> list:
        return [i for i in range(self.n()) if len(self.adjacency[i]) == 1]
        
    def find_leaves_solution(self) -> list:
        solution = [-1.] * self.n();
        temp = [-1.] * self.n();
        for leaf in self.leaves:
            solution[leaf] = 1;
        for iter in range(15000):
            solution, temp = temp, solution
            numerator = 0.
            for leaf in self.leaves:
                numerator += temp[leaf];
                solution[leaf] = len(self.leaves) - 1;
                for j in self.leaves:
                    solution[leaf] += self.distances[leaf][j] * temp[j];
            for leaf in self.leaves:
                solution[leaf] = numerator / solution[leaf];
        for leaf in self.leaves:
            solution[leaf] = 1 / solution[leaf];
        return solution;
        
    def find_solution(self) -> list:
        self.distances = [self.find_distances(root) for root in range(self.n())]
        self.leaves = self.find_leaves()
        if len(self.leaves) <= 1:
            return [0.] * self.n()
        solution = self.find_leaves_solution()
        denominator = 0.
        for leaf in self.leaves:
            denominator += 1 / solution[leaf]
        for i in range(self.n()):
            if solution[i] < 0:
                numerator = len(self.leaves) - 1
                for j in self.leaves:
                    numerator += self.distances[i][j] / solution[j];
                solution[i] = numerator / denominator
        return solution
    
    def solve(self) -> None:
        self.solution = self.find_solution()
    
    def find_ans(self, v: int) -> float:
        return self.solution[v]
 
 
def read_tree() -> Tree:
    edges = [tuple(map(lambda x: int(x) - 1, input().split())) for i in range(int(input()) - 1)]
    return Tree(edges)
 
 
def main() -> None:
    tree = read_tree()
    tree.solve()
    s = int(input()) - 1
    print(tree.find_ans(s))
 
 
if __name__ == '__main__':
    main()
