// Alice Sayutina (C) 2023

import java.io.PrintWriter
import java.util.Scanner

val max2: (Int, Int) -> Int = { acc, num -> Math.max(acc, num) }

class MaxTree(var array: IntArray) {
    private val n = array.size
    private val tree = IntArray(4 * n)
    private val modif = IntArray(4 * n)

    private fun build(no: Int, nl: Int, nr: Int) {
        if (nl == nr - 1) {
            tree[no] = array[nl]
        } else {
            val mi = nl + (nr - nl) / 2
            build(2 * no + 1, nl, mi)
            build(2 * no + 2, mi, nr)
            recalc(no)
        }
    }

    init {
        build(0, 0, n)
    }

    private fun push(no: Int, nl: Int, nr: Int) {
        tree[no] += modif[no]
        if (nr - nl > 1) {
            modif[2 * no + 1] += modif[no]
            modif[2 * no + 2] += modif[no]
        }
        modif[no] = 0
    }

    private fun recalc(no: Int) {
        tree[no] = Math.max(
            tree[2 * no + 1] + modif[2 * no + 1],
            tree[2 * no + 2] + modif[2 * no + 2],
        )
    }

    private fun increaseRec(no: Int, nl: Int, nr: Int, l: Int, r: Int, delta: Int) {
        if (l <= nl && nr <= r) {
            modif[no] += delta
            return
        }

        if (nr <= l || r <= nl)
            return

        val mi = nl + (nr - nl) / 2
        push(no, nl, nr)
        increaseRec(2 * no + 1, nl, mi, l, r, delta)
        increaseRec(2 * no + 2, mi, nr, l, r, delta)
        recalc(no)
    }

    private fun maxRec(no: Int, nl: Int, nr: Int, l: Int, r: Int): Int {
        push(no, nl, nr)

        if (l <= nl && nr <= r)
            return tree[no]

        if (nr <= l || r <= nl)
            return Int.MIN_VALUE / 2

        val mi = nl + (nr - nl) / 2
        return Math.max(
            maxRec(2 * no + 1, nl, mi, l, r),
            maxRec(2 * no + 2, mi, nr, l, r)
        )
    }

    fun descFindRightGEQ(no: Int, nl: Int, nr: Int, from: Int, num: Int): Int? {
        push(no, nl, nr)
        if (nr - 1 < from || tree[no] < num)
            return null

        if (nl == nr - 1)
            return nl

        val mi = nl + (nr - nl) / 2
        val leftResult = descFindRightGEQ(2 * no + 1, nl, mi, from, num)
        return leftResult ?: descFindRightGEQ(2 * no + 2, mi, nr, from, num)
    }

    fun descLeftFindGEQ(no: Int, nl: Int, nr: Int, from: Int, num: Int): Int? {
        push(no, nl, nr)
        if (from < nl || tree[no] < num)
            return null

        if (nl == nr - 1)
            return nl

        val mi = nl + (nr - nl) / 2
        val rightResult = descLeftFindGEQ(2 * no + 2, mi, nr, from, num)
        return rightResult ?: descLeftFindGEQ(2 * no + 1, nl, mi, from, num)
    }

    fun findLeftGEQ(from: Int, num: Int): Int = descLeftFindGEQ(0, 0, n, from, num) ?: -1
    fun findRightGEQ(from: Int, num: Int): Int = descFindRightGEQ(0, 0, n, from, num) ?: n
    fun increase(l: Int, r: Int, delta: Int)  = increaseRec(0, 0, n, l, r + 1, delta)
    fun maxOf(l: Int, r: Int): Int = maxRec(0, 0, n, l, r + 1)
    fun suffixMax(p: Int): Int = maxOf(p, n - 1)
    fun prefixMax(p: Int): Int = maxOf(0, p)
}

class M2Tree(arr1: IntArray, arr2: IntArray) {
    // arr1 - arr2 increases
    private var tree = MaxTree(arr1.zip(arr2).map { it.first - it.second }.toIntArray())
    private var baseAns = arr1.zip(arr2).sumOf { Math.min(it.first, it.second).toLong() }

    fun calc(): Long = baseAns

    fun countOnSegmentGE(l: Int, r: Int, num: Int): Int {
        val lprime = tree.findRightGEQ(l, num).coerceAtMost(r+1)
        return (r + 1) - lprime
    }

    fun countOnSegmentLE(l: Int, r: Int, num: Int) = (r - l + 1) - countOnSegmentGE(l, r, num + 1)

    fun incr1(l: Int, r: Int) {
        baseAns += countOnSegmentLE(l, r, -1)
        tree.increase(l, r, 1)
    }

    fun incr2(l: Int, r: Int) {
        baseAns += countOnSegmentGE(l, r, +1)
        tree.increase(l, r, -1)
    }
}

class Solution(array: IntArray) {
    private val n = array.size
    private var sum = array.sumOf { it.toLong() }
    private val mtree = MaxTree(array.clone())
    private val m2tree = M2Tree(
        array.runningReduce(max2).toIntArray(),
        array.reversed().runningReduce(max2).reversed().toIntArray()
    )

    fun solve() = m2tree.calc() - sum

    fun update(l: Int, r: Int) {
        sum += (r - l + 1)

        mtree.increase(l, r, 1)
        val mx = mtree.maxOf(l, r)

        val Lmax = mtree.prefixMax(l - 1)
        val Rmax = mtree.suffixMax(r + 1)

        if (mx > Rmax) {
            val Rposnext = mtree.findLeftGEQ(r, Rmax + 1)
            val RposEnd = mtree.findLeftGEQ(l - 1, mx)

            m2tree.incr2(RposEnd + 1, Rposnext)
        }

        if (mx > Lmax) {
            val Lposnext = mtree.findRightGEQ(l, Lmax + 1)
            val Lposend = mtree.findRightGEQ(r + 1, mx)

            m2tree.incr1(Lposnext, Lposend - 1)
        }
    }
}

fun main(args: Array<String>) {
    val sc = Scanner(System.`in`.buffered())
    val writer = PrintWriter(System.out.buffered())

    val n = sc.nextInt()
    val q = sc.nextInt()

    val array = IntArray(n) { sc.nextInt() }

    val solution = Solution(array)
    writer.println(solution.solve())

    for (i in 1..q) {
        val l = sc.nextInt() - 1
        val r = sc.nextInt() - 1
        solution.update(l, r)

        writer.println(solution.solve())
    }
    writer.flush()
}
