#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>

struct colors_t {
    int in_stack;
    int used;

    static colors_t create() {
        static int counter = 1;
        counter += 3;
        return colors_t{
            .in_stack = counter - 2,
            .used = counter - 1,
        };
    }
};

using namespace std;

vector<vector<int>> g;
vector<int> color;
vector<int> outs;

void dfs_directed(int v, bool& cycle_detected, const colors_t& schema, int ignore_vertex) {
    assert(color[v] != schema.in_stack && color[v] != schema.used);
    color[v] = schema.in_stack;

    for (int to : g[v]) {
        if (to == ignore_vertex) continue;
        if (color[to] == schema.used) continue;
        if (color[to] == schema.in_stack) {
            cycle_detected = true;
        } else {
            dfs_directed(to, cycle_detected, schema, ignore_vertex);
        }
    }

    color[v] = schema.used;
    outs.push_back(v);
}

int main() {
    std::ios::sync_with_stdio(false);

    int n, m;
    cin >> n >> m;

    g.resize(n);
    vector<vector<int>> grg(n);
    vector<char> has_edge_to(n);
    for (int i = 0; i < m; i++) {
        int a, b;
        cin >> a >> b;
        a--;
        b--;

        g[a].push_back(b);
        has_edge_to[b] = true;

        grg[a].push_back(b);
        grg[b].push_back(a);
    }

    vector<vector<int>> components;
    components.reserve(n);
    {
        std::vector<int> q;
        q.reserve(n);

        vector<char> used(n);
        for (int start = 0; start < n; start++) {
            if (used[start]) continue;
            q.clear();
            q.push_back(start);
            used[start] = true;

            for (size_t head = 0; head < q.size(); head++) {
                int v = q[head];
                for (int to : grg[v]) {
                    if (used[to]) continue;
                    used[to] = true;
                    q.push_back(to);
                }
            }

            components.push_back(q);

            // cerr << "found another component: ";
            // for (auto x : q) cerr << x << " ";
            // cerr << endl;
        }
    }

    std::vector<std::pair<int, int>> answer;
    answer.reserve(n);

    color.assign(n, 0);
    outs.reserve(n);
    for (auto& component : components) {

        // vector<int> chosen_start{};
        // for (int x : component) {
        //     if (!has_edge_to[x]) {
        //         chosen_start = {x};
        //         break;
        //     }
        // }

        // cerr << "chosen_start = " << (chosen_start.empty() ? -1 : chosen_start[0]) << endl;

        // const bool need_cycle = chosen_start.empty();
        vector<int> start_candidates = component;
        sort(start_candidates.begin(), start_candidates.end(), [&has_edge_to](int x, int y) {
            return (int) has_edge_to[x] < (int) has_edge_to[y];
        });
            // need_cycle
            //     ? component
            //     : chosen_start;

        bool success = false;

        // cerr << "process another component: ";
        // for (auto x : component) cerr << x << " ";
        // cerr << endl;

        for (int start : start_candidates) {
            outs.clear();
            bool has_cycle = false;
            const auto schema = colors_t::create();

            // cerr << "  try start = " << start << endl;

            for (int x : component) {
                if (color[x] == schema.used) continue;
                assert(color[x] != schema.in_stack);
                dfs_directed(x, has_cycle, schema, start);
            }

            reverse(outs.begin(), outs.end());
            auto& topsort = outs;

            // cerr << "    topsort: ";
            // for (auto x : topsort) cerr << x << " ";
            // cerr << endl;

            {
                auto start_iter = find(topsort.begin(), topsort.end(), start);
                assert(start_iter != topsort.end());

                // place start at the begin of topsort
                reverse(topsort.begin(), start_iter);
                reverse(topsort.begin(), next(start_iter));
            }

            // cerr << "    topsort after move start: ";
            // for (auto x : topsort) cerr << x << " ";
            // cerr << endl;

            bool topsort_suits = true;
            {
                static vector<size_t> position_of(n);
                for (size_t i = 0; i < topsort.size(); i++) {
                    int v = topsort[i];
                    position_of[v] = i;
                }
                for (int from : component) {
                    for (int to : g[from]) {
                        if (to == start) continue;
                        int from_pos = position_of[from];
                        int to_pos = position_of[to];
                        if (from_pos > to_pos) {
                            // cerr << "    topsort is bad because of edge " << from << " -> " << to << endl;
                            topsort_suits = false;
                        }
                    }
                }
            }
            if (!topsort_suits) {
                continue;
            }

            for (size_t i = 0; i + 1 < topsort.size(); i++) {
                int a = topsort[i];
                int b = topsort[i + 1];
                answer.emplace_back(a, b);
            }
            if (has_edge_to[start]) {
                answer.emplace_back(topsort.back(), topsort.front());
            }

            success = true;
            break;
        }

        if (!success) {
            // assert(need_cycle);
            cout << "-1\n";
            return 0;
        }
    }

    cout << answer.size() << '\n';
    for (auto [x, y] : answer) {
        cout << x + 1 << ' ' << y + 1 << '\n';
    }
}
