#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef double ld;
#define TIME (clock() * 1.0 / CLOCKS_PER_SEC)

const ld pi = acosl(-1.0);

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}

    inline T len2() const {
        return x * x + y * y;
    }

    inline ld len() const {
        return sqrtl(len2());
    }

    inline vect rotate() const {
        return vect(-y, x);
    }
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, const vect<T>& v) {
    return out << "(" << v.x << "u," << v.y << "u)";
}

template <typename T>
inline vect<T> operator+(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x + b.x, a.y + b.y);
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline vect<T> operator*(const vect<T>& a, T k) {
    return vect<T>(a.x * k, a.y * k);
}

template <typename T>
inline T operator*(const vect<T>& a, const vect<T>& b) {
    return a.x * b.x + a.y * b.y;
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

// _________ line (2D) ______________
template <typename T>
struct line {
    T a = 0;
    T b = 0;
    T c = 0;

    line() = default;
    line(T a_, T b_, T c_) : a(a_), b(b_), c(c_) {}
    line(const vect<T>& p, const vect<T>& dir) : a(-dir.y), b(dir.x), c(-(dir ^ p)) {}

    inline vect<T> normal() const {
        return vect<T>(a, b);
    }
};

template <typename T>
inline istream& operator>>(istream& in, line<T>& l) {
    return in >> l.a >> l.b >> l.c;
}

template <typename T>
inline ostream& operator<<(ostream& out, const line<T>& l) {
    //return out << l.a << "*x+" << l.b << "*y+" << l.c << "=0";
    return out << l.a << " " << l.b << " " << l.c;
}

template <typename T>
inline line<T> linePoints(const vect<T>& A, const vect<T>& B) {
    return line<T>(A, B - A);
}

const ld eps = 1e-9;

ld twice_area(const vector<vect<ld>>& v) {
    ld ans = 0.0;
    for (int i = 0; i < (int)v.size(); i++) {
        ans += (v[i] ^ v[(i + 1) % (int)v.size()]);
    }
    return abs(ans);
}

void draw_segment(const vect<ld>& a, const vect<ld>& b, ld pt = 1.0, const std::string& info = "", const std::string& suf = "") {
    cout << "pickup pencircle scaled " << pt << "pt;\n";
    if (info.empty()) {
        cout << "draw" << suf << " " << a << "--" << b << ";\n";
    } else {
        cout << "draw" << suf << " " << a << "--" << b << " " << info << ";\n";
    }
    cout << "\n";
}

void draw_point(const vect<ld>& p, ld pt = 4.0, const std::string& tex_label = "", const std::string& loc = "bot") {
    cout << "pickup pencircle scaled " << pt << "pt;\n";
    cout << "drawdot " << p << ";\n";
    if (!tex_label.empty()) {
        cout << "dotlabel." << loc << "(btex $" << tex_label << "$ etex, " << p << ");\n";
    }
    cout << "\n";
}

void paint_area(const vector<vect<ld>>& v) {
    int n = (int)v.size();

    for (int i = 0; i < n; i++) {
        cout << "path " << (char)(i + 'a') << ";\n";
        cout << (char)(i + 'a') << "=" << v[i] << "--" << v[(i + 1) % n] << ";\n";
    }

    cout << "path cyc;\n";
    cout << "cyc=buildcycle(";
    for (int i = 0; i < n; i++) {
        if (i > 0) {
            cout << ",";
        }
        cout << (char)(i + 'a');
    }
    cout << ");\n";

    cout << "fill cyc withcolor .8white;\n";

    cout << "\n";
}

void paint(const vector<vect<ld>>& v, int s) {
    int n = (int)v.size();

    ld mny = 1e9;
    ld mxy = -1e9;
    ld mnx = 1e9;
    ld mxx = -1e9;
    for (const auto& p : v) {
        mny = min(mny, p.y);
        mxy = max(mxy, p.y);
        mnx = min(mnx, p.x);
        mxx = max(mxx, p.x);
    }

    vector<vect<ld>> bottom;
    vect<ld> lg, rg;
    ld L = mny;
    ld R = mxy;
    for (int it = 0; it < 100; it++) {
        ld mid = (L + R) / 2;
        vector<pair<vect<ld>, int>> inters;
        for (int i = 0; i < n; i++) {
            auto p1 = v[i];
            auto p2 = v[(i + 1) % n];
            if (p1.y > p2.y) {
                swap(p1, p2);
            }
            if (p1.y - eps <= mid && mid <= p2.y + eps) {
                auto cur = p1 * ((p2.y - mid) / (p2.y - p1.y)) + p2 * ((mid - p1.y) / (p2.y - p1.y));
                inters.emplace_back(cur, (i + 1) % n);
            }
        }
        sort(inters.begin(), inters.end(), [&](const auto& p1, const auto& p2) {
            return p1.first.x < p2.first.x;
        });
        auto lp = inters[0].first;
        int st = inters[0].second;
        auto rp = inters[1].first;
        int en = inters.back().second;
        vector<vect<ld>> poly;

        poly.emplace_back(lp);
        for (int i = st; i != en; i = (i + 1) % n) {
            poly.emplace_back(v[i]);
        }
        poly.emplace_back(rp);

        lg = lp;
        rg = rp;
        bottom = poly;

        if (twice_area(poly) <= (ld)s) {
            L = mid;
        } else {
            R = mid;
        }
    }

    {
        auto pl = lg;
        ld shift = (mxx - mnx) / 10;
        pl.x = mnx - shift;
        auto pr = rg;
        pr.x = mxx + shift;
        draw_segment(pl, pr, 0.5, "dashed withdots scaled 0.5");
    }

    paint_area(bottom);

    int cnt = 0;
    for (const auto& p : bottom) {
        if (p.y >= lg.y - eps) {
            continue;
        }
        auto proj = vect<ld>(p.x, lg.y);
        draw_segment(p, proj, 0.5, "", "dblarrow");

        cnt++;
        auto md = (p + proj) * 0.5;
        cout << "label.lft(btex $d_{" << cnt << "}$ etex, " << md << ");\n";
    }

    for (int i = 0; i < n; i++) {
        draw_segment(v[i], v[(i + 1) % n]);
    }

    for (int i = 0; i < n; i++) {
        draw_point(v[i], (v[i].y >= lg.y - eps ? 3.0 : 5.0), to_string(i + 1));
    }

    draw_segment(lg, rg, 0.5);
}

inline vect<ld> rotate(const vect<ld>& p, ld phi) {
    return vect<ld>(p.x * cos(phi) - p.y * sin(phi), p.x * sin(phi) + p.y * cos(phi));
}

// (cos, -sin)
// (sin, cos)

vector<vect<ld>> rotate(const vector<vect<ld>>& v, ld phi) {
    vector<vect<ld>> res;
    for (const auto& p : v) {
        res.emplace_back(rotate(p, phi));
    }
    return res;
}

int main() {
#ifdef ONPC
    freopen("input", "r", stdin);
    freopen("output", "w", stdout);
#endif
    ios::sync_with_stdio(false); cin.tie(0); cout.tie(0);

    cout << fixed << setprecision(3);

    vector<vect<ld>> pt;
    //pt.emplace_back(vect<ld>(0, 0));
    //pt.emplace_back(vect<ld>(2, 0));
    //pt.emplace_back(vect<ld>(0, 2));
    pt.emplace_back(vect<ld>(-2, 1));
    pt.emplace_back(vect<ld>(-2, -3));
    pt.emplace_back(vect<ld>(-1, -4));
    pt.emplace_back(vect<ld>(0, -4));
    pt.emplace_back(vect<ld>(3, -3));
    pt.emplace_back(vect<ld>(4, -1));
    pt.emplace_back(vect<ld>(4, 0));
    pt.emplace_back(vect<ld>(-1, 2));

    cerr << twice_area(pt) << "\n";

    cerr << string(50, '_') << "\n";

    //vector<ld> rot_list = vector<ld>{0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5};
    vector<ld> rot_list = vector<ld>{1.15};

    for (int fig = 0; fig < (int)rot_list.size(); fig++) {
        //if (fig == 2) {
        //    break;
        //}

        cout << "beginfig(" << fig << ");\n";
        if (fig == 0) {
            cout << "u=1.0cm;\n\n";
        }

        ld phi = rot_list[fig];
        int s = 35;
        auto pt_rot = rotate(pt, phi);
        assert(s <= twice_area(pt_rot) + eps);
        paint(pt_rot, s);

        cout << "endfig;";
    }

    return 0;
}
