#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

typedef long long ll;
typedef long double ld;

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}

    inline T len2() const {
        return x * x + y * y;
    }

    inline ld len() const {
        return sqrtl(len2());
    }

    inline vect rotate() const {
        return vect(-y, x);
    }

    inline int half() const {
        return (y < 0 || (y == 0 && x < 0));
    }
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, const vect<T>& v) {
    return out << "(" << v.x << "u," << v.y << "u)";
}

template <typename T>
inline vect<T> operator+(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x + b.x, a.y + b.y);
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline vect<T> operator*(const vect<T>& a, T k) {
    return vect<T>(a.x * k, a.y * k);
}

template <typename T>
inline T operator*(const vect<T>& a, const vect<T>& b) {
    return a.x * b.x + a.y * b.y;
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

bool cmp(const vect<ll>& v1, const vect<ll>& v2) {
    int h1 = v1.half();
    int h2 = v2.half();
    if (h1 != h2) {
        return h1 < h2;
    }
    ll pr = (v1 ^ v2);
    if (pr != 0) {
        return pr > 0;
    }
    return v1.len2() < v2.len2();
}

vector<int> Divide(int s, int k) {
    if (k == 1) {
        return {s};
    }
    vector<int> pos;
    for (int i = 1; i < s; i++) {
        pos.emplace_back(i);
    }
    shuffle(pos.begin(), pos.end());
    sort(pos.begin(), pos.begin() + (k - 1));
    vector<int> sz;
    sz.emplace_back(pos[0]);
    for (int i = 0; i < k - 2; i++) {
        sz.emplace_back(pos[i + 1] - pos[i]);
    }
    sz.emplace_back(s - pos[k - 2]);
    return sz;
}

void PrintTest(vector<vect<ll>> v, ll s) {
    println(v.size(), s);
    int i = rnd.next((int)v.size());
    for (int j = 0; j < (int)v.size(); j++) {
        auto pt = v[(i + j) % (int)v.size()];
        println(pt.x, pt.y);
    }
}

constexpr ll kMaxX = 100'000'000LL;

void RandomScale(vector<vect<ll>>& v) {
    ll mxx = -1e18;
    ll mxy = mxx;
    ll mnx = -mxx;
    ll mny = -mxy;
    for (const auto& t : v) {
        mxx = max(mxx, t.x);
        mxy = max(mxy, t.y);
        mnx = min(mnx, t.x);
        mny = min(mny, t.y);
    }

    ll cx, cy;
    if (opt<bool>("noscale")) {
        cx = 1;
        cy = 1;
    } else {
        cx = rnd.next(1LL, (2 * kMaxX) / (mxx - mnx));
        cy = rnd.next(1LL, (2 * kMaxX) / (mxy - mny));
    }
    if (opt<bool>("eq")) {
        ll t = min(cx, cy);
        cx = t;
        cy = t;
    }

    mxx *= cx; mnx *= cx;
    mxy *= cy; mny *= cy;
    for (auto& t : v) {
        t.x *= cx;
        t.y *= cy;
    }

    ll dx, dy;
    if (opt<bool>("mxshift")) {
        if (rnd.next(2)) {
            dx = -kMaxX - mnx;
        } else {
            dx = kMaxX - mxx;
        }
        if (rnd.next(2)) {
            dy = -kMaxX - mny;
        } else {
            dy = kMaxX - mxy;
        }
    } else {
        dx = rnd.next(-kMaxX - mnx, kMaxX - mxx);
        dy = rnd.next(-kMaxX - mny, kMaxX - mxy);
    }

    mxx += dx; mnx += dx;
    mxy += dy; mny += dy;
    for (auto& t : v) {
        t.x += dx;
        t.y += dy;
    }
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);

    int nval = opt<int>("n");
    int t = opt<int>("t");
    int w = has_opt("w") ? opt<int>("w") : 0;

    vector<int> nums = Divide((nval / 2) - t, t);
    for (int& i : nums) {
        i = 2 * (i + 1);
    }

    // tests
    vector<pair<vector<vect<ll>>, ll>> tests;

    int it = 0;
    for (int n : nums) {
        it++;
        for (int c = 1; ; c *= 2) {
            vector<vect<ll>> sides;
            for (int x = -c; x <= c; x++) {
                for (int y = 0; y <= c; y++) {
                    if (x <= 0 && y == 0) {
                        continue;
                    }
                    if (gcd(abs(x), abs(y)) > 1) {
                        continue;
                    }
                    sides.emplace_back(vect<ll>(x, y));
                }
            }
            if (sides.size() < n / 2) {
                continue;
            }
            shuffle(sides.begin(), sides.end());
            sides.resize(n / 2);
            sort(sides.begin(), sides.end(), cmp);
            for (int i = 0; i < (n / 2); i++) {
                sides.emplace_back(sides[i] * (ll)(-1));
            }

            vector<vect<ll>> poly;
            vect<ll> cur(0LL, 0LL);
            for (int i = 0; i < n; i++) {
                poly.emplace_back(cur);
                cur = cur + sides[i];
            }

            RandomScale(poly);

            ll area = 0;
            for (int i = 0; i < n; i++) {
                area += poly[i] ^ poly[(i + 1) % n];
            }

            if ((area - 1) / 2 == 0) {
                continue;
            }
            ll s = rnd.wnext(1LL, ((area - 1) / 2), w);

            tests.emplace_back(poly, s);
            break;
        }
    }


    shuffle(tests.begin(), tests.end());
    println(tests.size());
    for (auto test : tests) {
        PrintTest(test.first, test.second);
    }

    return 0;
}
