#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

typedef long long ll;
typedef long double ld;

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}

    inline T len2() const {
        return x * x + y * y;
    }

    inline ld len() const {
        return sqrtl(len2());
    }

    inline vect rotate() const {
        return vect(-y, x);
    }

    inline int half() const {
        return (y < 0 || (y == 0 && x < 0));
    }
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, const vect<T>& v) {
    return out << "(" << v.x << "u," << v.y << "u)";
}

template <typename T>
inline vect<T> operator+(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x + b.x, a.y + b.y);
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline vect<T> operator*(const vect<T>& a, T k) {
    return vect<T>(a.x * k, a.y * k);
}

template <typename T>
inline T operator*(const vect<T>& a, const vect<T>& b) {
    return a.x * b.x + a.y * b.y;
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

template <typename T>
inline T operator==(const vect<T>& a, const vect<T>& b) {
    return a.x == b.x && a.y == b.y;
}

bool cmp(const vect<ll>& v1, const vect<ll>& v2) {
    int h1 = v1.half();
    int h2 = v2.half();
    if (h1 != h2) {
        return h1 < h2;
    }
    ll pr = (v1 ^ v2);
    if (pr != 0) {
        return pr > 0;
    }
    return v1.len2() < v2.len2();
}

void PrintTest(vector<vect<ll>> v, ll s) {
    println(v.size(), s);
    int i = rnd.next((int)v.size());
    for (int j = 0; j < (int)v.size(); j++) {
        auto pt = v[(i + j) % (int)v.size()];
        println(pt.x, pt.y);
    }
}

constexpr ll kMaxX = 100'000'000LL;

void RandomScale(vector<vect<ll>>& v) {
    ll mxx = -1e18;
    ll mxy = mxx;
    ll mnx = -mxx;
    ll mny = -mxy;
    for (const auto& t : v) {
        mxx = max(mxx, t.x);
        mxy = max(mxy, t.y);
        mnx = min(mnx, t.x);
        mny = min(mny, t.y);
    }

    ll cx, cy;
    if (opt<bool>("noscale")) {
        cx = 1;
        cy = 1;
    } else {
        cx = rnd.next(1LL, (2 * kMaxX) / (mxx - mnx));
        cy = rnd.next(1LL, (2 * kMaxX) / (mxy - mny));
    }
    if (opt<bool>("eq")) {
        ll t = min(cx, cy);
        cx = t;
        cy = t;
    }

    mxx *= cx; mnx *= cx;
    mxy *= cy; mny *= cy;
    for (auto& t : v) {
        t.x *= cx;
        t.y *= cy;
    }

    if (!opt<bool>("noscale")) {
        ll dx = rnd.next(-kMaxX - mnx, kMaxX - mxx);
        ll dy = rnd.next(-kMaxX - mny, kMaxX - mxy);

        mxx += dx;
        mnx += dx;
        mxy += dy;
        mny += dy;
        for (auto &t: v) {
            t.x += dx;
            t.y += dy;
        }
    }
}

ll get_area(const vector<vect<ll>>& p) {
    ll area = 0;
    for (int i = 0; i < (int)p.size(); i++) {
        area += p[i] ^ p[(i + 1) % (int)p.size()];
    }
    return area;
}

vector<vector<vect<ll>>> generated;
vector<vect<ll>> poly;
vector<vect<ll>> dir;

int glc;

void gen(int from, ll min_x, ll max_x) {
    if (poly.size() > 1 && poly.back() == poly[0]) {
        generated.emplace_back(poly);
        generated.back().pop_back();
        return;
    }
    if (from == dir.size()) {
        return;
    }
    do {
        auto new_pt = poly.back() + dir[from];
        ll new_min_x = min(min_x, new_pt.x);
        ll new_max_x = max(max_x, new_pt.x);
        if (new_pt.y < 0 || new_pt.y > glc || new_max_x - new_min_x > glc) {
            break;
        }
        if (new_pt.y == 0 && new_pt.x < 0) {
            break;
        }
        if (poly.size() >= 2 && ((poly.end()[-1] - poly.end()[-2]) ^ (new_pt - poly.back())) <= 0) {
            break;
        }
        if (poly.size() >= 2 && !(new_pt == poly[0]) && ((new_pt - poly.back()) ^ (poly[0] - new_pt)) <= 0) {
            break;
        }
        poly.emplace_back(new_pt);
        gen(from + 1, new_min_x, new_max_x);
        poly.pop_back();
    } while (false);
    gen(from + 1, min_x, max_x);
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);

    // tests
    vector<pair<vector<vect<ll>>, ll>> tests;

    for (int c = 1; ; c++) {
        generated.clear();
        poly.clear();

        dir.clear();
        for (int x = -c; x <= c; x++) {
            for (int y = -c; y <= c; y++) {
                if (x == 0 && y == 0) {
                    continue;
                }
                dir.emplace_back(x, y);
            }
        }
        sort(dir.begin(), dir.end(), cmp);

        poly.emplace_back(0LL, 0LL);
        glc = c;
        gen(0, 0, 0);

        vector<pair<vector<vect<ll>>, ll>> new_tests;
        ll tot_n = 0;
        for (const auto& p : generated) {
            auto cur_poly = p;
            RandomScale(cur_poly);
            ll area = get_area(cur_poly);
            ll up_s = (area - 1) / 2;
            if (up_s == 0) {
                continue;
            }
            ll s = rnd.next(1LL, up_s);
            tot_n += cur_poly.size();
            new_tests.emplace_back(cur_poly, s);
        }
        if (new_tests.size() > 10000) {
            shuffle(new_tests.begin(), new_tests.end());
            if (new_tests.size() > 10000) {
                new_tests.resize(10000);
            }
            tests = new_tests;
            break;
        }
        if (tot_n > 200000) {
            break;
        }
        tests = new_tests;
    }

    shuffle(tests.begin(), tests.end());
    println(tests.size());
    for (auto test : tests) {
        PrintTest(test.first, test.second);
    }

    return 0;
}
