#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

typedef long long ll;

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, const vect<T>& v) {
    return out << "[" << v.x << "," << v.y << "]";
}

template <typename T>
inline vect<T> operator+(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x + b.x, a.y + b.y);
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline vect<T> operator*(const vect<T>& a, T k) {
    return vect<T>(a.x * k, a.y * k);
}

template <typename T>
inline T operator*(const vect<T>& a, const vect<T>& b) {
    return a.x * b.x + a.y * b.y;
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

constexpr int kMaxT = 10'000;
constexpr int kMaxN = 200'000;
constexpr int kMaxX = 100'000'000;

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);

    int t = inf.readInt(1, kMaxT, "t");
    inf.readEoln();

    int sum_n = 0;

    for (int test = 0; test < t; ++test) {
        setTestCase(test + 1);

        int n = inf.readInt(3, kMaxN, "n");
        inf.readSpace();
        ll s = inf.readLong(1, (ll)kMaxX * (ll)kMaxX * 4LL, "s");
        inf.readEoln();

        sum_n += n;
        ensuref(sum_n <= kMaxN, "sum of n exceed %d", kMaxN);

        vector<vect<ll>> v(n);
        for (int i = 0; i < n; ++i) {
            v[i].x = inf.readInt(-kMaxX, kMaxX, format("x[%d]", i + 1));
            inf.readSpace();
            v[i].y = inf.readInt(-kMaxX, kMaxX, format("y[%d]", i + 1));
            inf.readEoln();
        }

        for (int i = 0; i < n; i++) {
            auto side1 = (v[(i + 1) % n] - v[i]);
            auto side2 = (v[(i + 2) % n] - v[(i + 1) % n]);
            ensuref((side1 ^ side2) > 0,
                    "polygon is not convex: (%d,%d)x(%d,%d) <= 0",
                    i + 1, ((i + 1) % n) + 1, ((i + 1) % n) + 1, ((i + 2) % n) + 1);
        }

        int best = 0;
        for (int i = 0; i < n; i++) {
            if (make_pair(v[i].y, v[i].x) < make_pair(v[best].y, v[best].x)) {
                best = i;
            }
        }

        for (int i = 1; i < n - 1; i++) {
            int x = (best + i) % n;
            int y = (best + i + 1) % n;

            auto side1 = (v[x] - v[best]);
            auto side2 = (v[y] - v[best]);
            ensuref((side1 ^ side2) > 0, "polygon is not convex");
        }

        ll area = 0;
        for (int i = 0; i < n; i++) {
            area += v[i] ^ v[(i + 1) % n];
        }
        ensuref(area > 0, "order should be counterclockwise");

        ensuref(area > s * 2LL, "s is not smaller than area");
    }

    inf.readEof();

    return 0;
}
