#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef double ld;
#define TIME (clock() * 1.0 / CLOCKS_PER_SEC)

const ld pi = acosl(-1.0);

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}

    inline T len2() const {
        return x * x + y * y;
    }

    inline ld len() const {
        return sqrtl(len2());
    }

    inline vect rotate() const {
        return vect(-y, x);
    }
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, const vect<T>& v) {
    return out << "(" << v.x << "u," << v.y << "u)";
}

template <typename T>
inline vect<T> operator+(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x + b.x, a.y + b.y);
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline vect<T> operator*(const vect<T>& a, T k) {
    return vect<T>(a.x * k, a.y * k);
}

template <typename T>
inline T operator*(const vect<T>& a, const vect<T>& b) {
    return a.x * b.x + a.y * b.y;
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

// _________ line (2D) ______________
template <typename T>
struct line {
    T a = 0;
    T b = 0;
    T c = 0;

    line() = default;
    line(T a_, T b_, T c_) : a(a_), b(b_), c(c_) {}
    line(const vect<T>& p, const vect<T>& dir) : a(-dir.y), b(dir.x), c(-(dir ^ p)) {}

    inline vect<T> normal() const {
        return vect<T>(a, b);
    }
};

template <typename T>
inline istream& operator>>(istream& in, line<T>& l) {
    return in >> l.a >> l.b >> l.c;
}

template <typename T>
inline ostream& operator<<(ostream& out, const line<T>& l) {
    //return out << l.a << "*x+" << l.b << "*y+" << l.c << "=0";
    return out << l.a << " " << l.b << " " << l.c;
}

template <typename T>
inline line<T> linePoints(const vect<T>& A, const vect<T>& B) {
    return line<T>(A, B - A);
}

const ld eps = 1e-9;

ld twice_area(const vector<vect<ld>>& v) {
    ld ans = 0.0;
    for (int i = 0; i < (int)v.size(); i++) {
        ans += (v[i] ^ v[(i + 1) % (int)v.size()]);
    }
    return abs(ans);
}

inline vect<ld> rotate(const vect<ll>& p, ld phi) {
    return vect<ld>(p.x * cos(phi) - p.y * sin(phi), p.x * sin(phi) + p.y * cos(phi));
}

// (cos, -sin)
// (sin, cos)

vector<vect<ld>> rotate(const vector<vect<ll>>& v, ld phi) {
    vector<vect<ld>> res;
    for (const auto& p : v) {
        res.emplace_back(rotate(p, phi));
    }
    return res;
}

mt19937 rnd(239);

ld paint(const vector<vect<ld>>& v, ll s) {
    int n = (int)v.size();

    ld mny = 1e9;
    ld mxy = -1e9;
    ld mnx = 1e9;
    ld mxx = -1e9;
    for (const auto& p : v) {
        mny = min(mny, p.y);
        mxy = max(mxy, p.y);
        mnx = min(mnx, p.x);
        mxx = max(mxx, p.x);
    }

    vector<vect<ld>> bottom;
    vect<ld> lg, rg;
    ld L = mny;
    ld R = mxy;
    for (int it = 0; it < 50; it++) {
        ld mid = (L + R) / 2;
        vector<pair<vect<ld>, int>> inters;
        for (int i = 0; i < n; i++) {
            auto p1 = v[i];
            auto p2 = v[(i + 1) % n];
            if (p1.y > p2.y) {
                swap(p1, p2);
            }
            if (p1.y - eps <= mid && mid <= p2.y + eps) {
                auto cur = p1 * ((p2.y - mid) / (p2.y - p1.y)) + p2 * ((mid - p1.y) / (p2.y - p1.y));
                inters.emplace_back(cur, (i + 1) % n);
            }
        }
        sort(inters.begin(), inters.end(), [&](const auto& p1, const auto& p2) {
            return p1.first.x < p2.first.x;
        });
        auto lp = inters[0].first;
        int st = inters[0].second;
        auto rp = inters[1].first;
        int en = inters.back().second;
        vector<vect<ld>> poly;

        poly.emplace_back(lp);
        for (int i = st; i != en; i = (i + 1) % n) {
            poly.emplace_back(v[i]);
        }
        poly.emplace_back(rp);

        lg = lp;
        rg = rp;
        bottom = poly;

        if (twice_area(poly) <= (ld)s) {
            L = mid;
        } else {
            R = mid;
        }
    }

    vector<ld> d;
    int cnt = 0;
    for (const auto& p : bottom) {
        if (p.y >= lg.y - eps) {
            continue;
        }
        d.emplace_back(lg.y - p.y);
    }

    ld mx = 0;
    ld sum = 0;
    for (auto x : d) {
        mx = max(mx, x);
        sum += x;
    }
    sum /= d.size();

    return mx - sum;
}

void solve() {
    int n;
    ll s;
    cin >> n >> s;
    s *= 2;
    vector<vect<ll>> v(n);
    for (int i = 0; i < n; i++) {
        cin >> v[i];
    }

    uniform_real_distribution<ld> uni(0, 2 * pi);
    ld sum = 0;

    const int N_ITER = 1000000;

    auto func = [&](ld phi) {
        return paint(rotate(v, phi), s);
    };

    for (int it = 0; it < N_ITER; ++it) {
        ld phi = uni(rnd);
        //assert(func(phi) > 0);

        sum += func(phi);
    }

    cout << sum / N_ITER << "\n";
}

int main() {
#ifdef ONPC
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0); cin.tie(0); cout.tie(0);

    cout << fixed << setprecision(10);

    int t;
    cin >> t;
    while (t--) {
        solve();
    }

    return 0;
}
