import java.io.*;
import java.util.*;

import static java.lang.Math.cos;
import static java.lang.Math.sin;

public class i_av {

    void solve() {
        int n = in.nextInt();
        long s = in.nextLong() * 2;
        Point[] p = readPoints(n);
        List<Event> events = generateEvents(n, p, s);
        double answer = 0;
        events.add(events.get(0));
        for (int i = 0; i < events.size(); i++) {
            Event cur = events.get(i), next = events.get((i + 1) % events.size());
            double fromAngle = cur.angle, toAngle = next.angle;
            if (i + 1 == events.size()) {
                toAngle += 2 * Math.PI;
            }
            answer -= cur.centerX * (sin(toAngle) - sin(fromAngle));
            answer -= cur.centerY * (-cos(toAngle) + cos(fromAngle));
        }
        answer += perimeter(p);
        out.println(answer / 2 / Math.PI);
    }

    private double perimeter(Point[] p) {
        double perim = 0;
        for (int i = 0; i < p.length; i++) {
            perim += p[i].distTo(p[(i + 1) % p.length]);
        }
        return perim;
    }

    private List<Event> generateEvents(int n, Point[] p, long s) {
        List<Event> events = new ArrayList<>();
        for (int rot = 0; rot < 2; rot++) {
            generateCCWEvents(n, p, s, events, rot == 1);
            for (int i = 0; i < n; i++) {
                p[i].y = -p[i].y;
            }
            for (int l = 0, r = n - 1; l < r; l++, r--) {
                Point tmp = p[l];
                p[l] = p[r];
                p[r] = tmp;
            }
        }
        Collections.sort(events, Comparator.comparingDouble(o -> o.angle));
        return events;
    }

    private void generateCCWEvents(int n, Point[] p, long s, List<Event> events, boolean isCW) {
        int j = 0;
        long sumX = p[0].x, sumY = p[0].y;
        int cnt = 1;
        for (int i = 0; i < p.length; i++) {
            while (true) {
                long addArea = vm(p[i], p[j], p[(j + 1) % n]);
                if (isCW && addArea < s || !isCW && addArea <= s) {
                    s -= addArea;
                    j = (j + 1) % n;
                    sumX += p[j].x;
                    sumY += p[j].y;
                    cnt++;
                } else {
                    double ratio = (double) s / addArea;
                    Event ev = calcNorm(p[i], p[j], p[(j + 1) % n], ratio);
                    if (isCW) {
                        ev.centerX = (double) sumX / cnt;
                        ev.centerY = (double) sumY / cnt;
                        ev.normY = -ev.normY;
                        ev.centerY = -ev.centerY;
                    } else {
                        ev.centerX = (double) (sumX - p[i].x) / (cnt - 1);
                        ev.centerY = (double) (sumY - p[i].y) / (cnt - 1);
                    }
                    ev.calcAngle();
                    events.add(ev);
                    break;
                }
            }
            s += vm(p[j], p[i], p[(i + 1) % n]);
            sumX -= p[i].x;
            sumY -= p[i].y;
            cnt--;
        }
    }

    private Event calcNorm(Point begin, Point segFirst, Point segSecond, double segRatio) {
        double dx = segFirst.x + (segSecond.x - segFirst.x) * segRatio - begin.x;
        double dy = segFirst.y + (segSecond.y - segFirst.y) * segRatio - begin.y;
        return new Event(dy, -dx);
    }

    long vm(Point a, Point b, Point c) {
        long x1 = b.x - a.x, y1 = b.y - a.y;
        long x2 = c.x - a.x, y2 = c.y - a.y;
        return x1 * y2 - x2 * y1;
    }

    class Point {
        int x, y;

        public Point(int x, int y) {
            this.x = x;
            this.y = y;
        }

        public double distTo(Point other) {
            long dx = x - other.x, dy = y - other.y;
            return Math.sqrt(dx * dx + dy * dy);
        }
    }

    class Event {
        double normX, normY;
        double centerX, centerY;
        double angle;

        public Event(double x, double y) {
            double norm = Math.sqrt(x * x + y * y);
            this.normX = x / norm;
            this.normY = y / norm;
        }

        public void calcAngle() {
            this.angle = Math.atan2(normY, normX);
        }
    }

    private Point[] readPoints(int n) {
        Point[] p = new Point[n];
        for (int i = 0; i < n; i++) {
            int x = in.nextInt(), y = in.nextInt();
            p[i] = new Point(x, y);
        }
        return p;
    }

    void run() {
        in = new FastScanner(System.in);
        out = new PrintWriter(System.out);
        int tests = in.nextInt();
        for (int t = 1; t <= tests; t++) {
            solve();
        }
        out.close();
    }

    FastScanner in;
    PrintWriter out;

    public static void main(String[] args) {
        new i_av().run();
    }

    public class FastScanner {
        public BufferedReader br;
        public StringTokenizer st;

        public FastScanner(InputStream in) {
            br = new BufferedReader(new InputStreamReader(in));
        }

        public FastScanner(String fileName) {
            try {
                br = new BufferedReader(new FileReader(fileName));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        public int nextInt() {
            return Integer.parseInt(next());
        }

        public String next() {
            while (st == null || !st.hasMoreElements()) {
                String line = null;
                try {
                    line = br.readLine();
                } catch (IOException e) {
                }
                st = new StringTokenizer(line);
            }
            return st.nextToken();
        }

        public long nextLong() {
            return Long.parseLong(next());
        }
    }
}
