#include <bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef long double ld;

const ld pi = acosl((ld)-1.0);

// _________ vect (2D) ______________
template <typename T>
struct vect {
    T x = 0;
    T y = 0;

    vect() = default;
    vect(T x_, T y_) : x(x_), y(y_) {}
};

template <typename T>
inline istream& operator>>(istream& in, vect<T>& v) {
    return in >> v.x >> v.y;
}

template <typename T>
inline ostream& operator<<(ostream& out, vect<T>& v) {
    return out << "[" << v.x << "," << v.y << "]";
}

template <typename T>
inline vect<T> operator-(const vect<T>& a, const vect<T>& b) {
    return vect<T>(a.x - b.x, a.y - b.y);
}

template <typename T>
inline T operator^(const vect<T>& a, const vect<T>& b) {
    return a.x * b.y - a.y * b.x;
}

ld divide(__int128 a, ll b) {
    return (ld)(a / b) + ((ld)(a % b) / (ld)b);
}

ld get_angle(const vect<ll>& a, const vect<ll>& b, const vect<ll>& c, ll triangle_area) {
    auto side1 = b - a;
    auto side2 = c - a;
    ll tot_area = abs(side1 ^ side2);

    __int128 vx = (__int128)side1.x * (tot_area - triangle_area) + (__int128)side2.x * triangle_area;
    __int128 vy = (__int128)side1.y * (tot_area - triangle_area) + (__int128)side2.y * triangle_area;

    vect<ld> v(divide(vx, tot_area), divide(vy, tot_area));

    return atan2l(v.y, v.x);
}

// integral of sin(x)
ld integrate_sin(ld x) {
    return -cosl(x);
}

// integral of cos(x)
ld integrate_cos(ld x) {
    return sinl(x);
}

void solve() {
    int n;
    ll s;
    cin >> n >> s;
    s *= 2;
    vector<vect<ll>> v(n);
    for (int i = 0; i < n; i++) {
        cin >> v[i];
    }
    vector<pair<ld, pair<ld, ld>>> events;
    for (int it = 0; it < 2; it++) {
        int p = 0;
        ll ps = 0;

        int k = 0;
        ll sx = 0;
        ll sy = 0;

        auto upd = [&](int i, int cf) {
            k += cf;
            sx += v[i].x * cf;
            sy += v[i].y * cf;
        };
        upd(0, 1);

        for (int i = 0; i < n; i++) {
            if (it == 0) {
                upd(i, -1);
            }

            while (abs(ps + (v[p] ^ v[(p + 1) % n]) + (v[(p + 1) % n] ^ v[i])) <= s - it) {
                ps += (v[p] ^ v[(p + 1) % n]);
                p = (p + 1) % n;
                upd(p, 1);
            }

            ll triangle_area = s - abs(ps + (v[p] ^ v[i]));
            ld angle = get_angle(v[i], v[p], v[(p + 1) % n], triangle_area);
            if (it == 1) {
                angle += pi;
                if (angle >= pi) {
                    angle -= 2 * pi;
                }
            }

            ld c_sin = -divide(sx, k);
            ld c_cos = divide(sy, k);

            events.emplace_back(angle, make_pair(c_sin, c_cos));

            ps -= (v[i] ^ v[(i + 1) % n]);

            if (it == 1) {
                upd(i, -1);
            }
        }
        reverse(v.begin(), v.end());
    }

    sort(events.begin(), events.end());

    ld ans = 0;
    for (int i = 0; i < (int)events.size(); i++) {
        ld phi = events[(i + 1) % (int)events.size()].first;
        ans += (events[i].second.first - events[(i + 1) % (int)events.size()].second.first) * integrate_sin(phi);
        ans += (events[i].second.second - events[(i + 1) % (int)events.size()].second.second) * integrate_cos(phi);
    }
    for (int i = 0; i < n; i++) {
        auto vc = v[(i + 1) % n] - v[i];
        ans += hypotl(vc.x, vc.y);
    }

    ans /= 2 * pi;

    cout << ans << "\n";
}

int main()
{
#ifdef ONPC
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0); cin.tie(0); cout.tie(0);

    cout << fixed << setprecision(20);

    int t;
    cin >> t;
    while (t--) {
        solve();
    }

    return 0;
}
