#include <algorithm>
#include <cstdint>
#include <cstdlib>
#include <functional>
#include <iostream>
#include <map>
#include <string>
#include <vector>

#include "testlib.h"

struct TestCase {
  std::vector<int64_t> limbs;
  int64_t total_limbs;
};

void OutputTestCases(std::vector<TestCase>& test_cases) {
  shuffle(test_cases.begin(), test_cases.end());
  std::cout << test_cases.size() << std::endl;
  for (auto& t : test_cases) {
    shuffle(t.limbs.begin(), t.limbs.end());
    std::cout << t.total_limbs << " " << t.limbs.size();
    for (auto l : t.limbs) {
      std::cout << " " << l;
    }
    std::cout << "\n";
  }
}

constexpr int64_t kMaxTotalLimbs = 1000'000'000;
constexpr int64_t kMaxTestCases = 1024;

void FillTotalLimbs(std::vector<TestCase>& test_cases,
                    int64_t approx_total_limbs, int64_t max_limbs) {
  if (approx_total_limbs == -1) {
    approx_total_limbs = kMaxTotalLimbs;
  }
  int64_t max_total_limbs = std::min(
      approx_total_limbs + max_limbs * 40 + test_cases.size() * 17,
      kMaxTotalLimbs);
  int64_t min_total_limbs = std::max(
      max_total_limbs / 3 * 2 - max_limbs * 40,
      int64_t{0});
  if (approx_total_limbs == 0) {
    min_total_limbs = 0;
    max_total_limbs = max_limbs * 4;
  }
  for (auto& t : test_cases) {
    t.total_limbs = rnd.next(min_total_limbs, max_total_limbs);
  }
}

void all_the_same(int argc, char* argv[]) {
  int64_t num_dims = atoi(argv[2]);
  int64_t min_limbs = atoi(argv[3]);
  int64_t max_limbs = atoi(argv[4]);
  int64_t approx_total_limbs = atoi(argv[5]);
  int64_t max_test_cases = argc == 7 ? atoi(argv[6]) : -1;
  if (max_test_cases == -1) {
    max_test_cases = kMaxTestCases;
  }

  std::vector<TestCase> test_cases;
  test_cases.reserve(max_test_cases);
  while (test_cases.size() < max_test_cases) {
    for (int64_t i = min_limbs; i <= max_limbs; ++i) {
      if (test_cases.size() == max_test_cases) {
        break;
      }
      test_cases.push_back(TestCase{std::vector<int64_t>(num_dims, i), -1});
    }
  }
  FillTotalLimbs(test_cases, approx_total_limbs, max_limbs);
  OutputTestCases(test_cases);
}

void from_range_unique_2d(int argc, char* argv[]) {
  int64_t min_limbs = atoi(argv[2]);
  int64_t max_limbs = atoi(argv[3]);
  int64_t approx_total_limbs = atoi(argv[4]);
  int64_t max_test_cases = argc == 6 ? atoi(argv[5]) : -1;
  if (max_test_cases == -1) {
    max_test_cases = kMaxTestCases;
  }

  std::vector<TestCase> test_cases;
  test_cases.reserve(max_test_cases);
  while (test_cases.size() < max_test_cases) {
    for (int64_t i = min_limbs; i <= max_limbs; ++i) {
      for (int64_t j = min_limbs; j < i; ++j) {
        if (test_cases.size() == max_test_cases) {
          break;
        }
        test_cases.push_back(TestCase{{i, j}, -1});
      }
    }
  }
  FillTotalLimbs(test_cases, approx_total_limbs, max_limbs);
  OutputTestCases(test_cases);
}

void from_range_unique_3d(int argc, char* argv[]) {
  int64_t min_limbs = atoi(argv[2]);
  int64_t max_limbs = atoi(argv[3]);
  int64_t approx_total_limbs = atoi(argv[4]);
  int64_t max_test_cases = argc == 6 ? atoi(argv[5]) : -1;
  if (max_test_cases == -1) {
    max_test_cases = kMaxTestCases;
  }

  std::vector<TestCase> test_cases;
  test_cases.reserve(max_test_cases);
  while (test_cases.size() < max_test_cases) {
    for (int64_t i = min_limbs; i <= max_limbs; ++i) {
      for (int64_t j = min_limbs; j < i; ++j) {
        for (int64_t k = min_limbs; k < j; ++k) {
          if (test_cases.size() == max_test_cases) {
            break;
          }
          test_cases.push_back(TestCase{{i, j, k}, -1});
        }
      }
    }
  }
  FillTotalLimbs(test_cases, approx_total_limbs, max_limbs);
  OutputTestCases(test_cases);
}

void from_range_two_same_3d(int argc, char* argv[]) {
  int64_t min_limbs = atoi(argv[2]);
  int64_t max_limbs = atoi(argv[3]);
  int64_t approx_total_limbs = atoi(argv[4]);
  int64_t max_test_cases = argc == 6 ? atoi(argv[5]) : -1;
  if (max_test_cases == -1) {
    max_test_cases = kMaxTestCases;
  }

  std::vector<TestCase> test_cases;
  test_cases.reserve(max_test_cases);
  while (test_cases.size() < max_test_cases) {
    for (int64_t i = min_limbs; i <= max_limbs; ++i) {
      for (int64_t j = min_limbs; j < i; ++j) {
        for (int64_t k : {i, j}) {
          if (test_cases.size() == max_test_cases) {
            break;
          }
          test_cases.push_back(TestCase{{i, j, k}, -1});
        }
      }
    }
  }
  FillTotalLimbs(test_cases, approx_total_limbs, max_limbs);
  OutputTestCases(test_cases);
}

void from_range_random_mix(int argc, char* argv[]) {
  int64_t min_limbs = atoi(argv[2]);
  int64_t max_limbs = atoi(argv[3]);
  int64_t approx_total_limbs = atoi(argv[4]);
  int64_t max_test_cases = argc == 6 ? atoi(argv[5]) : -1;
  if (max_test_cases == -1) {
    max_test_cases = kMaxTestCases;
  }

  std::vector<TestCase> test_cases;
  test_cases.reserve(max_test_cases);
  while (test_cases.size() < max_test_cases) {
    int64_t num_dims = rnd.next(1, 100);
    if (num_dims != 1) {  // 1% for 1 dimension is enough
      // 9% for 2 case, other 90 for 3.
      num_dims = num_dims < 10 ? 2 : 3;
    }
    test_cases.push_back(TestCase{std::vector<int64_t>(num_dims, -1), -1});
    for (int64_t i = 0; i < num_dims; ++i) {
      test_cases.back().limbs[i] = rnd.next(min_limbs, max_limbs);
    }
  }
  FillTotalLimbs(test_cases, approx_total_limbs, max_limbs);
  OutputTestCases(test_cases);
}

using Generator = std::function<void(int argc, char* argv[])>;

std::map<std::string, Generator> generators = {
    {"all_the_same", all_the_same},                      //
    {"from_range_unique_3d", from_range_unique_3d},      //
    {"from_range_unique_2d", from_range_unique_2d},      //
    {"from_range_two_same_3d", from_range_two_same_3d},  //
    {"from_range_random_mix", from_range_random_mix},    //
};

int main(int argc, char* argv[]) {
  registerGen(argc, argv, 1);
  auto mode = std::string(argv[1]);
  if (auto it = generators.find(mode); it != generators.end()) {
    it->second(argc, argv);
    return 0;
  }
  std::cout << std::string("FAIL!\n");
  return 1;
}