#include "testlib.h"
#include <bits/stdc++.h>
using namespace std;

#define MULTITEST_ENABLED 0

const int MAX_N = 2e5;
const int MAX_A = 2e5;

struct Test {
    vector<int> a;

    void print() {
        cout << a.size() << endl;
        for (int i = 0; i < (int)a.size(); i++) cout << a[i] << " \n"[i + 1 == (int)a.size()];
    }
};

struct Multitest {
    vector<Test> tests;

    void print() {
        if (MULTITEST_ENABLED) {
            cout << tests.size() << endl;
            for (auto &t : tests) {
                t.print();
            }
        } else {
            assert(tests.size() == 1);
            tests[0].print();
        }
    }
};

template<typename T>
T opt_default(string name, T def) {
    return __testlib_opts.count(name) ? opt<T>(name) : def;
}

auto gens = [](){
    map<string, function<Multitest()>> res;

    auto gen_multitest = [&](function<Test()> &&gen_test) {
        return [gen_test{move(gen_test)}]() {
            if (MULTITEST_ENABLED && opt_default<int>("stress", 0)) {
                vector<Test> tests;
                int sum_n = 0;
                while (1) {
                    auto test = gen_test();
                    sum_n += test.a.size();
                    if (sum_n > MAX_N) break;
                    tests.push_back(test);
                }
                return Multitest({tests});
            } else {
                return Multitest({{gen_test()}});
            }
        };
    };
    res["rand"] = gen_multitest([&](){
        int n = opt<int>("n");
        int mn = opt_default<int>("min", 1);
        int mx = opt_default<int>("max", MAX_A);
        vector<int> a(n);
        for (int i = 0; i < n; i++) {
            a[i] = rnd.next(mn, mx);
        }
        return Test{a};
    });

    return res;
}();

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    string type = opt<string>("type");
    if (gens.count(type)) {
        gens[type]().print();
    } else {
        cerr << "unexpected test type = " << type << endl;
        return 1;
    }
}
